#include <SPI.h>
#include <Ethernet.h>

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network.
// gateway and subnet are optional:
byte mac[] = { 
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(130,226,233,210);

EthernetServer server(23);

String inputString = "";         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete

void setup() {
  // Initialize pins:
  pinMode(30, OUTPUT);
  pinMode(32, OUTPUT);
  pinMode(36, OUTPUT);
  pinMode(38, OUTPUT);
  pinMode(42, OUTPUT);
  pinMode(44, OUTPUT);
  digitalWrite(30, LOW);
  digitalWrite(32, LOW); 
  digitalWrite(36, LOW);
  digitalWrite(38, LOW); 
  digitalWrite(42, LOW);
  digitalWrite(44, LOW); 
  
  // initialize serial:
  Serial.begin(115200);
  
  // reserve 200 bytes for the inputString:
  inputString.reserve(200);
  /*while(!Serial) {
    ; // Wait for serial
  }*/
  
  Ethernet.begin(mac, ip);
  ip = Ethernet.localIP();
  server.begin();
}

void loop() {
  EthernetClient client = server.available();
  if(client) {
    char c;
    inputString = "";
    while((c = client.read()) != -1) {
      inputString += c;
      // if the incoming character is a newline, set a flag
      // so the main loop can do something about it:
      if (c == '\n' || inputString.length() == 199) {
        stringComplete = true;
        break;
      }
    }
  }
  
  
  // print the string when a newline arrives:
  if (stringComplete) {
    inputString.toUpperCase();
    inputString.trim();
    if(Serial) {
      Serial.println(inputString);
    }
    boolean isValid = getIsValid(inputString);
    String msg;
    if(isValid) {
      int outletNr = getOutletNr(inputString);
      boolean onState = getOnState(inputString);
      setOutletState(outletNr, onState);
      
      String stateString = (onState) ? "ON" : "OFF";
      String outletNrString = String(outletNr);
      msg = "Turned outlet " + outletNrString + " " + stateString;
    }
    else {
      msg = "Invalid command. Format has to be <OutletNr>-<State>, fx. 1-ON";
    }
    // Write message:
    if(Serial) {
      Serial.println(msg);
    }
    if(client) {
      msg += "\n";
      char msgBytes[msg.length()+1];
      msg.toCharArray(msgBytes,msg.length()+1);
      for(int i = 0; i < msg.length()+1; i++) {
        server.write(msgBytes[i]);
      }
    }
    // clear the string:
    inputString = "";
    stringComplete = false;
  }
}

/*
  SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 */
void serialEvent() {
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read(); 
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n' || inputString.length() == 199) {
      stringComplete = true;
      break;
    } 
  }
}

boolean getIsValid(String str) {
  int dashPosition = str.indexOf("-");
  if(dashPosition < 1) {
    return false;
  }
 if(str.substring(0,dashPosition+1).toInt() == 0) {
   return false;
 }
 if(str.substring(dashPosition+1) != "ON" && str.substring(dashPosition+1) != "OFF") {
   return false;
 }
 return true; 
}

int getOutletNr(String str) {
  int dashPosition = str.indexOf("-");
  if(dashPosition < 1) {
    return 0;
  }
  return (int)str.substring(0,dashPosition).toInt();
}
boolean getOnState(String str) {
  int dashPosition = str.indexOf("-");
  if(dashPosition < 1) {
    return false;
  }
  if(str.substring(dashPosition+1) == "ON") {
    return true;
  }
  return false;
}

void setOutletState(int outlet, boolean onState) {
  // ###### ON: ######
  if(onState) {
    switch(outlet) {
      case 1:
        // DEV 1:
        digitalWrite(30, HIGH);
        delay(500);
        digitalWrite(30, LOW);
        break;
      case 2:
        // DEV 2:
        digitalWrite(36, HIGH);
        delay(500);
        digitalWrite(36, LOW);
        break;
      case 3:
        // DEV 3:
        digitalWrite(42, HIGH);
        delay(500);
        digitalWrite(42, LOW);
        break;
    }
  }
  // ###### OFF: ######
  else {
    switch(outlet) {
      case 1:
        // DEV 1:
        digitalWrite(32, HIGH);
        delay(500);
        digitalWrite(32, LOW);
        break;
      case 2:
        // DEV 2:
        digitalWrite(38, HIGH);
        delay(500);
        digitalWrite(38, LOW);
        break;
      case 3:
        // DEV 3:
        digitalWrite(44, HIGH);
        delay(500);
        digitalWrite(44, LOW);
        break;
    }
  }
}
